//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright(C) 2002-2016 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-------------------------------------------------------------------------
//
// Purpose        : 
//
// Special Notes  :
//
// Creator        : Dave Baur
//
// Creation Date  : 
//
//
//
//
//-------------------------------------------------------------------------

#include <Xyce_config.h>

#include <N_IO_ActiveOutput.h>
#include <N_IO_OutputMgr.h>
#include <N_ANP_SweepParam.h>

namespace Xyce {
namespace IO {

ActiveOutput::ActiveOutput(OutputMgr &output_manager)
  : outputManager_(output_manager)
{
  outputManager_.pushActiveOutputters();
}

ActiveOutput::~ActiveOutput()
{
  outputManager_.popActiveOutputters();
}

void
ActiveOutput::add(PrintType::PrintType print_type, Analysis::Mode analysis_mode)
{
  outputManager_.addActiveOutputter(print_type, analysis_mode);
}

void
ActiveOutput::resetIndex()
{
  outputManager_.resetIndex();
}

void
ActiveOutput::add(Parallel::Machine comm, Analysis::Mode analysis_mode)
{
  outputManager_.prepareOutput(comm, analysis_mode);
}

void
ActiveOutput::setStepSweepVector(
  const Analysis::SweepVector & sweep_vector)
{
  outputManager_.setStepSweepVector(sweep_vector);
}

void
ActiveOutput::setDCSweepVector(
  const Analysis::SweepVector & sweep_vector)
{
  outputManager_.setDCSweepVector(sweep_vector);
}

} // namespace IO
} // namespace Xyce
